;                   PTR_X : SuperBASIC function returning
;                            the real part of a complex number (x)
;                   PTR_Y : SUPERBASIC function returning
;                           imaginary part of a complex number (y)
;                   RTP_R : SUPERBASIC function returning the module
;                           of a complex number (radius)
;                   RTP_T : SUPERBASIC function returning the argument
;                           of a complex number (theta)

;                   usage :

;                           real_part      = PTR_X(argument,module)
;                           imaginary_part = PTR_Y(argument,module)
;                           module         = RTP_R(real_part,imaginary_part)
;                           argument       = RTP_T(real_part,imaginary_part)


bv_rip              equ       $58
ri_add              equ       $0a
ri_sub              equ       $0c
ri_mult             equ       $0e
ri_div              equ       $10
ri_neg              equ       $14
ri_dup              equ       $16
ri_cos              equ       $18
ri_sin              equ       $1a
ri_atan             equ       $24
ri_root             equ       $28


                    move.w    bp_init,a0
                    lea       TABLE,a1
                    jsr       (a0)
                    rts

PTR_X               moveq     #ri_cos,d7
                    bra.s     DEB_PTR
PTR_Y               moveq     #ri_sin,d7

DEB_PTR             bsr.s     SAISIE_PARAMETRES
                    bne.s     ERREUR_BP

                    move.b    8(a6,a1.l),d5       The module must be
                    bmi.s     ERREUR_BP           positive...

                    move.w    d7,d0
                    jsr       (a0)                COS(t) or SIN(t)
                    bne.s     ERREUR

                    moveq     #ri_mult,d0         r*COS(t) or r*SIN(t)
                    jsr       (a0)

                    bra.s     SORTIE

RTP_R               bsr.s     SAISIE_PARAMETRES
                    bne.s     ERREUR_BP

                    move.l    2(a6,a1.l),d5
                    or.l      8(a6,a1.l),d5
                    beq.s     ERREUR_BP           x = y = 0

                    move.w    bv_chrix,a2         Reserve 6 bytes on the
                    moveq     #6,d1               RI stack
                    jsr       (a2)
                    move.l    bv_rip(a6),a1

                    bsr.s     CARRE               x^2
                    bne.s     ERREUR

                    move.w    0(a6,a1.l),d5       Save the value
                    move.l    2(a6,a1.l),d6
                    add.l     #6,a1

                    bsr.s     CARRE               y^2
                    bne.s     ERREUR

                    sub.l     #6,a1               Recover the value
                    move.w    d5,0(a6,a1.l)       saved beforehand
                    move.l    d6,2(a6,a1.l)

                    moveq     #ri_add,d0          x^2+y^2
                    jsr       (a0)
                    bne.s     ERREUR

                    moveq     #ri_root,d0         SQRT(x^2+y^2)
                    jsr       (a0)

SORTIE              move.l    a1,bv_rip(a6)       Point to the result
                    moveq     #2,d4               Type = floating point
ERREUR              rts
ERREUR_BP           moveq     #err_bp,d0
                    rts

CARRE               moveq     #ri_dup,d0          TOS^2
                    jsr       (a0)
                    moveq     #ri_mult,d0
                    jmp       (a0)

SAISIE_PARAMETRES   move.w    ca_gtfp,a0
                    jsr       (a0)
                    bne.s     ERREUR
                    sub.w     #2,d3               The function only accepts
                    bne.s     ERREUR              two parameters

                    move.w    ri_exec,a0          Arithmetic Operation
                    rts

RTP_T               bsr.s     SAISIE_PARAMETRES
                    bne.s     ERREUR_BP

                    move.l    #$6487ed51,d7       Mantissa of PI
                    move.l    8(a6,a1.l),d6          "     "  y
                    move.l    2(a6,a1.l),d5          "     "  x
                    bne.s     DIFF_ZERO
                    tst.l     d6
                    beq.s     ERREUR_BP           x = y = 0

                    add.l     #6,a1
                    move.w    #$0801,0(a6,a1.l)   +PI/2
                    move.l    d7,2(a6,a1.l)

                    tst.l     d6
                    bpl.s     SORTIE

                    moveq     #ri_neg,d0          -PI/2
                    jsr       (a0)

                    bra.s     SORTIE

DIFF_ZERO           moveq     #ri_div,d0          y/x
                    jsr       (a0)
                    bne.s     ERREUR

                    moveq     #ri_atan,d0         ATAN(y/x)
                    jsr       (a0)
                    bne.s     ERREUR

                    tst.l     d5
                    bpl.s     SORTIE              Dials 1 & 4

                    sub.l     #6,a1
                    move.w    #$0802,0(a6,a1.l)   PI
                    move.l    d7,2(a6,a1.l)

                    moveq     #ri_add,d0          Dial 2
                    tst.l     d6
                    bpl.s     VECTEUR
                    moveq     #ri_sub,d0          Dial 3
VECTEUR             jsr       (a0)

                    bra.s     SORTIE

TABLE               dc.w      0,0,4
                    dc.w      PTR_X-*
                    dc.b      5,"PTR_X"
                    dc.w      PTR_Y-*
                    dc.b      5,"PTR_Y"
                    dc.w      RTP_R-*
                    dc.b      5,"RTP_R"
                    dc.w      RTP_T-*
                    dc.b      5,"RTP_T"
                    dc.w      0

                    END
